% Finding final pH using a value of C0_KB
% Output in "pH_store" provide a vector of pH calues corresponding with the input experimental data 
% 
clear all
close all
clc

% Loading experimental data
data = xlsread('Experimental_data_sorted.xlsx');
C_Actot_data =  data(:,1)*1000; % mol/m^3
pH_data =       data(:,2);

Troom = 20 + 273.15; % K

rho_20 = Density(Troom); % kg/m3

C_H_20 = 10.^(-pH_data).*rho_20; % mol/m3


format shortE

% Initial conditions of the pretreatment
pH0 = 6;
V0 = 50E-6;         % m^3
N0_RAc = V0*81.07;  % mol 
N0_HAc = 0;         % mol
N0_Ac = 0;          % mol
N0_H = V0*1000*10^(-pH0);   % mol
N0_OH = V0*1000*10^(-14 + pH0);% mol
N0_HB = 0;      % mol
C0_B = 5.558;   % mol/m3  (buffer in potassium salt form)
N0_B = V0*C0_B; % mol
T0 = Troom;         % K

j = length(C_Actot_data);


C_Actot_exp = C_Actot_data; % mol/m3
pH_exp = pH_data;

for i=1:j

C_Ac_tot = C_Actot_exp(i);                          % mol/m3
Keq_HAc_20 = 1000*10.^-4.768;                       % mol/m3
pKa_HB = 5.1755;                                    % molality basis
Keq_HB = 10.^(-pKa_HB)*1000;                        % mol/m3

a_term = 1;  
b_term = Keq_HAc_20 + Keq_HB + C0_B;                % mol/m3
c_term = Keq_HAc_20.*(C0_B + Keq_HB - C_Ac_tot);    % (mol/m3)^2 
d_term = -C_Ac_tot .* Keq_HAc_20 .* Keq_HB;         % (mol/m3)^3
polynome = [a_term b_term c_term d_term];
r = roots(polynome);

% select real, positive solution
C_H_20end = max(r);                                 % mol/m3
pH_final = -log10(C_H_20end/1000); 
pH_store(i,1) = pH_final; 

end

%% Model definition
function dC = dCdt(t,C,A1,Ea1,Tset)

% Variables identification
N_RAc   = C(1);
N_HAc   = C(2);
N_Ac    = C(3);
N_H     = C(4);
N_OH    = C(5);
N_HB    = C(6);
N_B     = C(7);
T       = C(8);
V       = C(9);

% Mole amount to molar concentration
C_RAc   = N_RAc/V;
C_HAc   = N_HAc/V;
C_Ac    = N_Ac/V;
C_H     = N_H/V;
C_OH    = N_OH/V;
C_HB    = N_HB/V;
C_B     = N_B/V;

% Parameters
R = 8.314471e-3;        % kJ/mol K
k = 1000;               % value high enough for equilibration but not slowing down simulation; unit depends on use
c_Heating = 2.8172;     % J/(K s)
rho = Density(T);       % kg/m^3
rhodT = DiffDensity(T); % kg/m^3 K
Cp = HeatCapacity(T);   % J/kg K

% Equilibrium constants and pKa values
Keq_HAc = Dissconst_HAc(T);           % mol/kg
pKa_HAc = -log10(Keq_HAc);            % molality based
pKa_HB  = 5.1755 + pKa_HAc - 4.758;   % molality based
Keq_HB = 10.^(-pKa_HB)*1000;          % Eq. constants based on HAc equilibrium
Kw = EQconst_H2O(T);                  % (mol/kg)^2
pKa_H2O = -log10(Kw);                 % molality based

% Irreversible reaction in mol.m-3.s-1
r_RAc = C_RAc * C_H * A1 * exp(-Ea1/(R*T));

% Equilibrium reactions in mol.m-3.s-1
r_eq_HAc = k * (C_HAc - (C_H*C_Ac/(Keq_HAc*rho)));              
r_eq_H2O = k * (1 - (C_H*C_OH/(Kw*rho^2)));
r_eq_HB  = k * (C_HB - (C_H*C_B/(Keq_HB*rho)));

% Differential Equations 
dC(1) = -r_RAc*V;                       % mol/s
dC(2) = (r_RAc - r_eq_HAc)*V;
dC(3) = r_eq_HAc*V;
dC(4) = (r_eq_HAc + r_eq_H2O + r_eq_HB)*V;
dC(5) = r_eq_H2O*V;
dC(6) = (- r_eq_HB)*V;
dC(7) = (r_eq_HB)*V;
dC(8) = c_Heating / Cp * (Tset-T);
dC(9) = (c_Heating / Cp * (Tset-T))*(-V/rho*rhodT);
dC = dC';
end

function rhodT = DiffDensity(T)
% T in Kelvin, return drho/dt  in kg/(m^3 K)

% Parameters
A1 = 0.6884103;
A2 = -2.44970115E-3;
A3 = 3.7060667E-6;
A4 = -2.11062995E-9;
A5 = -1.12273895E-13;

% Derivative of Density calculation
rhodT = 18.015 .*(A1 + 2.*A2.*T + 3.*A3.*T.^2 + 4.*A4.*T.^3 + 5.*A5.*T.^4);

end

function rho = Density(T)
% T in Kelvin, return density of pure water in kg/m^3

% Parameters
A0 = -13.418392;
A1 = 0.6884103;
A2 = -2.44970115E-3;
A3 = 3.7060667E-6;
A4 = -2.11062995E-9;
A5 = -1.12273895E-13;

% Density calculation
rho = 18.015 .* (A0 + A1.*T + A2.*T.^2 + A3.*T.^3 + A4.*T.^4 + A5.*T.^5);

end

function Cp = HeatCapacity(T)
% T in Kelvin, return Specific Heat Capacity of pure water in J/kg K

% General Parameters
Tc = 647.096;   % K
R = 8.314471;   % J/mol K
MW = 18.015;    % g/mol

% Specific heat capacity parameters
A = 0.255980; B = 12.54595; C = -31.40896;
D = 97.76650; E = -145.4236; F = 87.0185;

% Specific Heat Capacity calculation (adapted for J/kg K)
Tau = 1- T/Tc;
Cp = (R .* (A./Tau + B + C.*Tau + D.*Tau.^2 + E.*Tau.^3 + F.*Tau.^4 ) ./ MW)*1000;

end

function Kw = EQconst_H2O(T)
% T in Kelvin, return Equilibrium constant of water, molality based

% Equilibrium constant - Pure Water
Kw = exp(-13445.9./T - 22.4773*log(T) + 140.932);

end

function Keq_HAc = Dissconst_HAc(T)
% T in Kelvin, return acid dissociation constant of acetic acid, molality based

% Parameters
dS_er0 =    -7.42856;   % cal/(mol*K)
dS_r0 =     -22.1280;   % cal/(mol*K)
dH_r0 =     -108.273;   % cal/mol
a =         -59.9424;   % -
b =         0.112254;   % - 
T_ref =     298.15;     % K
R =         1.9872;     % cal/(mol*K)

% Acid dissociation constant - Acetic acid
K0 = dS_er0.*(T_ref - T - 218.*(1 - exp(exp(-12.714 + 0.01875.*T) - 7.84.*10^-4 ...
    + (T - T_ref)./219))) - dH_r0 + T.*dS_r0 + T.*a.*(log(T./T_ref) - 1 + ...
    (T_ref./T)) + b/2.*(T - T_ref).^2;
Keq_HAc = exp(K0./(R.*T));

end