% Finding values for A2 and Ea2
clear all
close all
clc

% Loading experimental data
data = xlsread('Experimental_data_sorted.xlsx');
C_Actot_data =  data(:,1)*1000; % mol/m^3
pH_data =       data(:,2);
t_heat_data =   data(:,3);      % min
t_pret_data =   data(:,4);      % min
t_process = (t_heat_data + t_pret_data)*60; % s
T_set = data(:,6) + 273.15;     % K
Troom = 20 + 273.15;            % K

rho_20 = Density(Troom); % kg/m3
C_H_20 = 10.^(-pH_data).*rho_20; % mol/m3

pKa_HAc_473 = -log10(Dissconst_HAc(473));
pKa_HB_473 = 5.1755 + pKa_HAc_473 - 4.758;   % molality based

format shortE

% Initial conditions of the pretreatment
pH0 = 6;
V0 = 50E-6;         % m^3
N0_RAc = V0*88.07;  % mol 
N0_HAc = 0;         % mol
N0_Ac = 0;          % mol
N0_H = V0*1000*10^(-pH0);   % mol
N0_OH = V0*1000*10^(-14 + pH0);% mol
N0_HB = 0;          % mol
C0_B =  5.558;      % mol/m3  (buffer in potassium salt form)
N0_B = V0*C0_B;     % mol
T0 = Troom;         % K

C0 = [N0_RAc N0_HAc N0_Ac N0_H N0_OH N0_HB N0_B T0 V0];

% Initial iteration values and options for the optimization
x0 = [180330.2991 77.2221];

options = optimset('Display','iter','MaxFunEvals',1000,'MaxIter',100,'TolX',1e-1,'TolFun',1e-4);

% Finding values for A2 and Ea2 (called here A2 and Ea2)
[xopt,fopt] = fminsearch(@(x) Residual(x,C_Actot_data,t_process,T_set,C0), x0, options);

A1 = xopt(1);
Ea1 = xopt(2);
SSE_val = fopt(1);     

Results = [A1;Ea1;SSE_val];

% Representation
disp(['Experiments: A1 = ' num2str(xopt(1)) ' ((mol/(m^3*s)   Ea1 = ' num2str(xopt(2)) ' kJ/mol        SSE = ' num2str(fopt(1))])

%% 

function SSE = Residual(x,C_Actot_data,t_process,T_set,C0)

j = length(C_Actot_data);   %number of experiments

% Defining Parameters
A1 = x(1);      % m^3/(mol*s)
Ea1 = x(2);     % kJ/mol
C_Actot_exp = C_Actot_data; % mol/m3

SSE_store = zeros([j 1]);
C_Actot_store = zeros([j 1]);

for i=1:j

t0 = 0; tend = t_process(i);% s
tspan = t0:10:tend;      % s
Tset = T_set(i) ; % K

% Solving the numerical model with ode15s
opt = odeset('RelTol',1e-11, 'AbsTol',1e-11);
[t,C] = ode15s(@(t,C) dCdt(t,C,A1,Ea1,Tset),tspan,C0,opt);

V0 = 50E-6;            % m3
C_RAc = C(:,1)./V0;    % mol/m3
C_HAc = C(:,2)./V0;    % mol/m3
C_Ac = C(:,3)./V0;     % mol/m3
C_H = C(:,4)./V0;      % mol/m3
C_HB = C(:,5)./V0;     % mol/m3
C_B = C(:,6)./V0;      % mol/m3
pH = -log10(C_H./1000); 
T = C(:,8) - 273.15;   % oC
V = C(:,9);            % m3

C_Actot_sim = C_HAc(end) + C_Ac(end);

C_Actot_store(i) = C_HAc(end) + C_Ac(end);
SSE_store(i) = (C_Actot_sim - C_Actot_exp(i)).^2;

writematrix ([C_Actot_data C_Actot_store],'Experimental and simulated C_Actot in mM'); % Exports raw data

end

SSE = sum(SSE_store);

for j = 51
    %% Plot results 

%     % Acetic species
% figure('Name','Straathof2023','NumberTitle','off'); set(gcf,'color','w');
% subplot(3,2,1); xlabel('Time (min)');axis([0 inf 0 inf]);
% yyaxis left; plot(t/60, C_RAc,'k', 'LineWidth',2), 'o'; set(gca,'FontSize',10,'YColor','k');  
% ylabel('c_{RAc} (mol.m{^-^3})'); 
% yyaxis right; plot(t/60, C_Ac+C_HAc,'b--','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
% ylabel('c_{Ac}^{total} (mol.m{^-^3})'); 
% legend('RAc','Actot');
% 
%     % pH
% subplot(3,2,2); xlabel('Time (min)'); axis([0 inf 0 inf]);
% yyaxis left; plot(t/60,pH(:,1),'b--','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
% ylabel('pH');
% legend('pH');

    % Temperature
% subplot(3,2,3); xlabel('Time (min)'); axis([0 inf 0 inf]);
% yyaxis left; plot(t/60, T, 'b--','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
% ylabel('T (^oC)');
% legend('Temperature');

    % Volume
% subplot(3,2,4); xlabel('Time (min)'); axis([0 inf 0 inf]);
% yyaxis left; plot(t/60, V*1E6, 'b--','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
% ylabel('V (mL)');
% legend('Reaction volume');
 
%% Export results to excel 
%xlswrite('Batch progess curve.xlsx',[i,j]); % Exports raw data

end

end

%% Model definition
function dC = dCdt(t,C,A1,Ea1,Tset)

% Variables identification
N_RAc   = C(1);
N_HAc   = C(2);
N_Ac    = C(3);
N_H     = C(4);
N_OH    = C(5);
N_HB    = C(6);
N_B     = C(7);
T       = C(8);
V       = C(9);

% Mole amount to molar concentration
C_RAc   = N_RAc/V;
C_HAc   = N_HAc/V;
C_Ac    = N_Ac/V;
C_H     = N_H/V;
C_OH    = N_OH/V;
C_HB    = N_HB/V;
C_B     = N_B/V;

% Parameters
R = 8.314471e-3;        % kJ/mol K
k = 1000;               % value high enough for equilibration but not slowing down simulation; unit depends on use
c_Heating = 2.8172;     % J/(K s)
rho = Density(T);       % kg/m^3
rhodT = DiffDensity(T); % kg/m^3 K
Cp = HeatCapacity(T);   % J/kg K

% Equilibrium constants and pKa values
Keq_HAc = Dissconst_HAc(T);           % mol/kg
pKa_HAc = -log10(Keq_HAc);            % molality based
pKa_HB  = 5.1755 + pKa_HAc - 4.758;   % molality based
Keq_HB = 10.^(-pKa_HB);               % mol/kg  
Kw = EQconst_H2O(T);                  % (mol/kg)^2
pKa_H2O = -log10(Kw);                 % molality based

% Irreversible reaction in mol.m-3.s-1
r_RAc = C_RAc * A1 * exp(-Ea1/(R*T));

% Equilibrium reactions in mol.m-3.s-1
r_eq_HAc = k * (C_HAc - (C_H*C_Ac/(Keq_HAc*rho)));              
r_eq_H2O = k * (1 - (C_H*C_OH/(Kw*rho^2)));
r_eq_HB  = k * (C_HB - (C_H*C_B/(Keq_HB*rho)));

% Differential Equations 
dC(1) = -r_RAc*V;                                   % dNi/dt in mol/s
dC(2) = (r_RAc - r_eq_HAc)*V;                       % dNi/dt in mol/s
dC(3) = r_eq_HAc*V;                                 % dNi/dt in mol/s
dC(4) = (r_eq_HAc + r_eq_H2O + r_eq_HB)*V;          % dNi/dt in mol/s
dC(5) = r_eq_H2O*V;                                 % dNi/dt in mol/s
dC(6) = (- r_eq_HB)*V;                              % dNi/dt in mol/s
dC(7) = (r_eq_HB)*V;                                % dNi/dt in mol/s
dC(8) = c_Heating / Cp * (Tset-T);                  % dT/dt in K/s    
dC(9) = (c_Heating / Cp * (Tset-T))*(-V/rho*rhodT); % dV/dt in m3/s
dC = dC';
end

function rhodT = DiffDensity(T)
% T in Kelvin, return drho/dt  in kg/(m^3 K)

% Parameters
A1 = 0.6884103;
A2 = -2.44970115E-3;
A3 = 3.7060667E-6;
A4 = -2.11062995E-9;
A5 = -1.12273895E-13;

% Derivative of Density calculation
rhodT = 18.015 .*(A1 + 2.*A2.*T + 3.*A3.*T.^2 + 4.*A4.*T.^3 + 5.*A5.*T.^4);

end

function rho = Density(T)
% T in Kelvin, return density of pure water in kg/m^3

% Parameters
A0 = -13.418392;
A1 = 0.6884103;
A2 = -2.44970115E-3;
A3 = 3.7060667E-6;
A4 = -2.11062995E-9;
A5 = -1.12273895E-13;

% Density calculation
rho = 18.015 .* (A0 + A1.*T + A2.*T.^2 + A3.*T.^3 + A4.*T.^4 + A5.*T.^5);

end

function Cp = HeatCapacity(T)
% T in Kelvin, return Specific Heat Capacity of pure water in J/kg K

% General Parameters
Tc = 647.096;   % K
R = 8.314471;   % J/mol K
MW = 18.015;    % g/mol

% Specific heat capacity parameters
A = 0.255980; B = 12.54595; C = -31.40896;
D = 97.76650; E = -145.4236; F = 87.0185;

% Specific Heat Capacity calculation (adapted for J/kg K)
Tau = 1- T/Tc;
Cp = (R .* (A./Tau + B + C.*Tau + D.*Tau.^2 + E.*Tau.^3 + F.*Tau.^4 ) ./ MW)*1000;

end

function Kw = EQconst_H2O(T)
% T in Kelvin, return Equilibrium constant of water, molality based

% Equilibrium constant - Pure Water
Kw = exp(-13445.9./T - 22.4773*log(T) + 140.932);

end

function Keq_HAc = Dissconst_HAc(T)
% T in Kelvin, return acid dissociation constant of acetic acid, molality based

% Parameters
dS_er0 =    -7.42856;   % cal/(mol*K)
dS_r0 =     -22.1280;   % cal/(mol*K)
dH_r0 =     -108.273;   % cal/mol
a =         -59.9424;   % -
b =         0.112254;   % - 
T_ref =     298.15;     % K
R =         1.9872;     % cal/(mol*K)

% Acid dissociation constant - Acetic acid
K0 = dS_er0.*(T_ref - T - 218.*(1 - exp(exp(-12.714 + 0.01875.*T) - 7.84.*10^-4 ...
    + (T - T_ref)./219))) - dH_r0 + T.*dS_r0 + T.*a.*(log(T./T_ref) - 1 + ...
    (T_ref./T)) + b/2.*(T - T_ref).^2;
Keq_HAc = exp(K0./(R.*T));

end