%% Temperature fitting
clear all
close all
clc

%% Heating
% Experimental Data of Temperature and Time
data1 = [0 21;5 34;10 62;15 82;20	96;25	108;30	118;35	126;40	133;...
45	140;50	146;55	152;60	157;65	162;70	166;75	170;80	173;85	176;...
90	178];

t1 = data1(:,1)*60; % time, s
TC1 = data1(:,2);   % Temp, C
T1 = TC1+273.15;    % Temp, K
Tset1 = 180+273.15; % Temp. set, K

% Initial conditions and options
C01 = 200;      % J/m3 K s
options = optimset('MaxIter',50);

% Optimization function
[Copt1,fopt1] = fminsearch(@(C) ResidualT(C,T1,t1,Tset1), C01, options);

% Displaying value of C
disp(['Optimal C value for Heating is = ' num2str(Copt1) ' J/(K s)'])


%% Heating + Pretreatment Data (for representation)
data1 = [0 21;5 34;10 62;15 82;20	96;25	108;30	118;35	126;40	133;...
45	140;50	146;55	152;60	157;65	162;70	166;75	170;80	173;85	176;...
90	178;95	179;100	179;105	179;110	179;115	179;120	179;125	179;130	179;...
135	180;140	179;145	178;150	179;155	178;160	180;165	179;170	178;175	179;...
180	180;185	179;190	178;195	179;200	180;205	180;210	180];

t1 = data1(:,1)*60; % time, s
TC1 = data1(:,2);   % Temp, C
T1 = TC1+273.15;    % Temp, K
Tset1 = 180+273.15; % Temp. set, K


%% Cooling
% Experimental Data of Temperature and Time
%data2 = xlsread('Tprofile.xlsx',2);
data2 = [215 178.7;220 150.9;225 123.0;230 101.1;235 83.6;240	70.3;...
245	59.3;250	51.0;255	44.5;260	39.2;265	35.2;270	32.1;...
275	29.7;280	27.7;285	26.3;290	25.2;295	24.4;300	23.7];

t2 = data2(:,1)*60; % Time, s
TC2 = data2(:,2);   % Temp, C
T2 = TC2+273.15;    % Temp, K
Tset2 = 20+273.15;  % Temp, set, K

% Initial conditions and options
C02 = 300;      % J/m3 K s
options = optimset('MaxIter',50);

% Optimization function
[Copt2,fopt2] = fminsearch(@(C) ResidualT(C,T2,t2,Tset2), C02, options);

% Displaying value of C
disp(['Optimal C value for Cooling is = ' num2str(Copt2) ' J/(K s)'])

%% Graphical Representation

% Vector definition for time
tspan1 = 0:300:12600; % s
tspan1 = tspan1';
tspan2 = 12900:300:18000; % s
tspan2 = tspan2';

% Parameters Heating
rho1 = Density(T1);   % kg/m3
Cp1 = HeatCapacity(T1)*1000; % J/kg K
T01 = T1(1);
t01 = t1(1);
Topt1 = Tset1-((Tset1-T01).*exp(-Copt1./(Cp1).*(tspan1-t01)));

% Parameters Cooling
rho2 = Density(T2);   % kg/m3
Cp2 = HeatCapacity(T2)*1000;% J/kg K
Tset2 = 20+273.15;  % K
T02 = T2(1);
t02 = tspan2(1);
Topt2 = Tset2-((Tset2-T02).*exp(-Copt2./(Cp2).*(tspan2-t02)));

% Combine Heating and Cooling
T = [T1;T2]-273.15; % C
t = [t1;t2]./60;    % min
tspan = [tspan1;tspan2]./60; % min
Topt = [Topt1;Topt2]-273.15; % C

% Graph
plot(tspan,Topt,'-','LineWidth',2,'Color',[1 0.6 0])
hold on
plot(t,T,'.','MarkerSize',18,'Color',[0 0.4 0.7])
xlabel('Time, min','FontSize',16);
ylabel('Temperature, C','FontSize',16);
ylim([0 200]);
legend('Simulation','Experimental','FontSize',14);


%% Functions used in this script
% Sum of Squared Residuals (difference between simulated and experimental
% values of internal temperature T
function SSR = ResidualT(C,T,t,Tset)

% Parameters
rho = Density(T);   % kg/m3
Cp = HeatCapacity(T).*1000;% J/kg K
T0 = T(1);
t0 = t(1);

% Simulation and Experimental values
tsim = (-(Cp) ./ C).*log((Tset - T)./(Tset - T0)) + t0;
texp = t;

% Sum of the Squared Residuals
SSR = sum((tsim - texp).^2);

end

% Function for density of pure water (Gmehling et al. 2019)
function rho = Density(T)% T in Kelvin, return rho in kg/m3

% Parameters
A0 = -13.418392;
A1 = 0.6884103;
A2 = -2.44970115E-3;
A3 = 3.7060667E-6;
A4 = -2.11062995E-9;
A5 = -1.12273895E-13;

% Density calculation
rho = 18.015 .* (A0 + A1.*T + A2.*T.^2 + A3.*T.^3 + A4.*T.^4 + A5.*T.^5);

end

% Function for Specific Heat Capacity of pure water (Gmehling et al. 2019)
function Cp = HeatCapacity(T)% T in Kelvin, return Cp in J/g K

% General Parameters
Tc = 647.096;   % K
R = 8.314471;   % J/mol K
MW = 18.015;    % g/mol

% Specific heat capacity parameters
A = 0.255980; B = 12.54595; C = -31.40896;
D = 97.76650; E = -145.4236; F = 87.0185;

% Specific Heat Capacity calculation
Tau = 1- T/Tc;
Cp = R .* (A./Tau + B + C.*Tau + D.*Tau.^2 + E.*Tau.^3 + F.*Tau.^4 ) ./ MW;

end